function B = im2bw_sauvola_fast(g,w,k,show_image)
% Efficiently Binarize a Grayscale Image using Sauvola's Method
% INPUT:
% g------------> Graysacale Image (2D matrix)
% w------------> Window Size (scalar)
% k------------> Parameter [0,0.5] (scalar)
% show_image---> Shows Binarization Output (logical scalar)
%
% OUTPUT:
% B---> Binary Image (2D logical matrix)
%
% Based on:
%
% [1] Sauvola, Jaakko, and Matti Pietikinen. "Adaptive document image
% binarization." Pattern Recognition 33.2 (2000): 225-236.
%
% [2] Shafait, Faisal, Daniel Keysers, and Thomas M. Breuel. "Efficient
% implementation of local adaptive thresholding techniques using integral
% images." Document Recognition and Retrieval XV 6815.1 (2008): 681510.
%
% Author: Zohaib Khan, University of Western Australia.
% email: zohaib@csse.uwa.edu.au
% 
% Please Cite:
%
% [1] Z. Khan, F. Shafait and A. Mian, "Hyperspectral Imaging for Ink
% Mismatch Detection", in 12th International Conference on Document
% Analysis and Recognition (ICDAR), pp. 877-881, 2013.

if nargin<4
    show_image = false;
    if nargin<3
        k = 0.15;
        if nargin<2
            w = 32;
        end
    end
end

g = double(padarray(g, [w/2 w/2])); % Zero Padding at Image Borders
R = 128;                            % Maximum Value for Standard Deviation (128 for grayscale image)
I = cumsum(cumsum(g),2);            % Integral Image of Pixel Intensities
I2 = cumsum(cumsum(g.^2),2);        % Integral Image of Squared Pixel Intensities
[M N] = size(g);                    % Image Dimensions

% Mean
m = (I(1+w:M,1+w:N)+I(1:M-w,1:N-w)-I(1+w:M,1:N-w)-I(1:M-w,1+w:N))/w^2;
% Standard Deviation
s = sqrt((I2(1+w:M,1+w:N)+I2(1:M-w,1:N-w)-I2(1+w:M,1:N-w)-I2(1:M-w,1+w:N))/w^2-m.^2);
% Threshold
t = m.*(1+k*(s/R-1));
% Binary Image
B = false(M-w,N-w); B(g(1+w/2:M-w/2,1+w/2:N-w/2)>t) = true;

    if(show_image)
        subplot(1,2,1), imshow(g(1+w/2:M-w/2,1+w/2:N-w/2),[])
        subplot(1,2,2), imshow(B)
        waitforbuttonpress
    end
end